 /*
 +---------------------------------------------------------------------------+
 | Facebook Microsoft Vista Sidebar Gadget                                   |
 +---------------------------------------------------------------------------+
 | Copyright (c) 2007 Facebook, Inc.                                         |
 | All rights reserved.                                                      |
 |                                                                           |
 | Redistribution and use in source and binary forms, with or without        |
 | modification, are permitted provided that the following conditions        |
 | are met:                                                                  |
 |                                                                           |
 | 1. Redistributions of source code must retain the above copyright         |
 |    notice, this list of conditions and the following disclaimer.          |
 | 2. Redistributions in binary form must reproduce the above copyright      |
 |    notice, this list of conditions and the following disclaimer in the    |
 |    documentation and/or other materials provided with the distribution.   |
 |                                                                           |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR      |
 | IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES |
 | OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.   |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,          |
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  |
 | NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, |
 | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     |
 | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       |
 | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF  |
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.         |
 +---------------------------------------------------------------------------+
 | Developer: Amine Nebri - facebook id anebri@yahoo.com                     |
 +---------------------------------------------------------------------------+
*/

/*
 * AjaxResponse is a custom class used to store the result of an http post call
 * including the url, response text, status, statusText, and readyState properties
 * of the XmlHttpRequest object.
 */
function AjaxResponse(url)
{
    this.url = url; // url of the HTTP post
    this.length = 0; // length of the response
    this.responseContent = "";// actual content of the response
    this.status = ""; // HTTP status
    this.statusText = ""; // HTTP status text
    this.readyState = ""; // HTTP ready state
    
    /*
     * Returns an info string with all the properties of the class instance.
     */
    this.stringValue = function()
    {
        return "responseContent="+this.responseContent+",url="+url+",status="+this.status+",statusText="+this.statusText+",readyState="+this.readyState;
    }
    
    /*
     * Returns the facebook error code if any. This is done by looking
     * for 2 properties in the response. error_code first and then
     * fb_error.code.
     *
     * For optimization purposes, we assume that if the length of the
     * response is greater than GadgetSetting.MaximumErrorResponseLength
     * than the response does not contain an error. This saves us the time
     * of having to parse a large response to find out there's no error in
     * it.
     */
    this.getErrorCode = function()
    {        
        var result = "";

        if (this.length <= GadgetSetting.MaximumErrorResponseLength)
        {   
            try
            {
                result = this.responseContent.error_code;
            }
            catch (ex)
            {
                try
                {
                    result = this.responseContent.fb_error.code;
                }
                catch (ex)
                {
                    //swallow
                }
            }
        }
        
        if (result == undefined)
            result = "";

	    return result;
    }
    
    /*
     * Returns the facebook error message if any. This is done by looking
     * for 2 properties in the response. error_msg first and then
     * fb_error.message.
     *
     * For optimization purposes, we assume that if the length of the
     * response is greater than GadgetSetting.MaximumErrorResponseLength
     * than the response does not contain an error. This saves us the time
     * of having to parse a large response to find out there's no error in
     * it.
     */
    this.getErrorMessage = function()
    {
        var result = "";

        if (this.length <= GadgetSetting.MaximumErrorResponseLength)
        {
            try
            {
                result = this.responseContent.error_msg;
            }
            catch (ex)
            {
                try
                {
                    result = this.responseContent.fb_error.message;
                }
                catch (ex)
                {
                    //swallow
                }
            }
        }
        
        if (result == undefined)
            result = "";
        
	    return result;
    }
    
    /*
     * Returns true if this instance has an actual response. false otherwise
     */
    this.hasResponse = function()
    {
        if (this.responseContent == undefined || this.responseContent == "")
           return false;
           
        return true;
    }
}

/*
 * The AjaxUtil singleton class implements functions for posting HTTP messages
 * and pinging urls. HTTP Posts return a custom class called AjaxResponse
 * that stores all the info about the http response.
 * AjaxUtil leverages the jquery javascript library
 */
AjaxUtil = new function()
{  
    /*
     * Sends an HTTP POST mesage to the provided url and parses the response
     * based on the value of format. Possible values are JSON, XML, other (Text).
     * We use jquery to post the HTTP message.
     * This function returns an AjaxResponse object.
     */
    this.doPost = function(url, format)
    {
        Util.debug("doPost: url="+url);
        
        // We start by creating an empty response object
        var response = this.createAjaxResponse(url);
        
        try
        {
            // We use jquery to invoke the http post synchronously
            var request = $.ajax({type: "POST", url: url, async: false});

            // We populate the response object with the results
            response.readyState = request.readyState;
            response.status = request.status;
            response.statusText = request.statusText;

            // We assume the call was not successfull if readyState is not 4
            // and the status is not 200
            if (request.readyState != 4 || request.status != 200)
                Util.error("doPost", "HTTP Send Call Failed. readyState="+request.readyState + " status="+request.status);
                
            if (Util.isDebug())
            {
                if (Util.startsWith(request.responseText, "<!DOCTYPE html"))
                    Util.debug("doPost: responseText="+request.responseText.substring(0, Math.min(request.responseText.length, 200))+" ...");
                else
                    Util.debug("doPost: responseText="+request.responseText);
                    
                Util.debug("doPost: Status="+response.status+" StatusText="+response.statusText);
                Util.debug("doPost: response length is "+request.responseText.length+" bytes");
            }
            
            // We populate the responseConent based on format
            if (format == undefined)
                response.responseContent = request.responseText;
            else
            {
                Util.debug("doPost: format="+format);
                response.length = request.responseText.length;
            
                switch(format)
                {
                    case "JSON":
                        response.responseContent = Util.getJSON(request.responseText);
                        Util.debug("doPost: response format is JSON");
                        break    
                    case "XML":
                        response.responseContent = request.responseXML.xml;
                        Util.debug("doPost: response format is XML");
                      break
                    default:
                      response.responseContent = request.responseText;
                      Util.debug("doPost: response format is Text");
                }
            }
        
            return response;
        }
        catch (ex)
        {
            Util.error("doPost", ex);
        }
    }
    
    /*
     * Pings the url defined in GadgetSetting.PingUrl and returns true
     * if it is successfull. false otherwise.
     * We assume ping is successful if the response status is 200.
     */
    this.ping = function()
    {
        try
        {
            var response = this.doPost(GadgetSetting.PingUrl);
                
            if (response.status == 200)
            {
                Util.debug("ping: Successful");
                return true;
            }
            
            Util.debug("ping: Failed response:"+response.stringValue());
            
            return false;
         }
         catch (ex)
         {
            Util.debug("ping: Failed response:"+response.stringValue());
            
            return false;
         }
    }
    
    /*
     * Returns an empty AjaxResponse object.
     */
    this.createAjaxResponse = function(url)
    {
        return new AjaxResponse(url);
    }
}
