 /*
 +---------------------------------------------------------------------------+
 | Facebook Microsoft Vista Sidebar Gadget                                   |
 +---------------------------------------------------------------------------+
 | Copyright (c) 2007 Facebook, Inc.                                         |
 | All rights reserved.                                                      |
 |                                                                           |
 | Redistribution and use in source and binary forms, with or without        |
 | modification, are permitted provided that the following conditions        |
 | are met:                                                                  |
 |                                                                           |
 | 1. Redistributions of source code must retain the above copyright         |
 |    notice, this list of conditions and the following disclaimer.          |
 | 2. Redistributions in binary form must reproduce the above copyright      |
 |    notice, this list of conditions and the following disclaimer in the    |
 |    documentation and/or other materials provided with the distribution.   |
 |                                                                           |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR      |
 | IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES |
 | OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.   |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,          |
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  |
 | NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, |
 | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     |
 | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       |
 | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF  |
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.         |
 +---------------------------------------------------------------------------+
 | Developer: Amine Nebri - facebook id anebri@yahoo.com                     |
 +---------------------------------------------------------------------------+
*/

/*
 * Photo is the class representing the facebook photo entity.
 * Photo has several properties but for performance reasons we
 * only care about 3 properties src_big, link, and caption. The
 * photo instance is created from a JSON object.
 */
function Photo(photo)
{
    this.bigSource = photo.src_big; // photo's large url
    this.link = photo.link; // public url of the photo on the facebook site
    this.caption = photo.caption; // photo's caption
    
    /*
     * Returns an info string with all the properties of the class instance.
     */
    this.stringValue = function()
    {
        return "bigSource=" + this.bigSource + ",link="+this.link + ",caption=" + this.caption; 
    }
}

/*
 * Notification is the class representing the facebook notifications
 * entity.
 * notifications has several properties but for performance reasons we
 * only care about 2 properties unreadPokes and mostRecentPokes. The
 * Notification instance is created from a JSON object.
 */
function Notification(notification)
{
    this.unreadPokes = notification.pokes.unread; // number of unread pokes
    this.mostRecentPokes = notification.pokes.most_recent; // number of most recent pokes
    
    /*
     * Returns an info string with all the properties of the class instance.
     */
    this.stringValue = function()
    {
        return "unreadPokes=" + this.unreadPokes + ",mostRecentPokes=" + this.mostRecentPokes; 
    }
}

/*
 * FacebookAPI is the core class of the gadget. It is a singleton class that implements
 * the REST interface to facebook's REST server for implementing the functionality required
 * for the Vista Sidebar gadget including retrieving and posting data such as
 * uploading photos, and displaying a slideshow, and number of pokes.
 * Functions starting with self are private functions meant to only be called from
 * within the class. FacebookAPI maintains its own state using the Vista Gadget Settings.
 * Most function start by restoring the state. Photos are retrieved using FQL.
 * JSON is the format used to send and receive data. We're using version 1.0 of the facebook
 * API. We rely on an open source library to generate the MD5 signature.
 *
 * Go to http://developers.facebook.com/documentation.php?v=1.0 for more details
 * on the facebook API
 */
FacebookAPI = new function()
{
    var self = this; // Used to implement private functions
    this.secret = GadgetSetting.Secret; // Secret key required for all calls. Initial value is set to the key of the dev account
    this.token; // Token used to authenticate
    this.session; // session key
    this.uid; // The user id of the facebook user
    this.photoUploadAlbumLink = ""; // Link property of the pending photos album
    this.photoUploadAlbumName = ""; // Name property of the pending photos album
    this.delay = GadgetSetting.MillisecondsBeforeRetry;
    
    /*
     * Opens a new browser window to the facebook page
     * that allows the user to get a token code.
     */
    this.doGetAuthenticationToken = function()
    {
        window.open(self.getAuthenticationTokenUrl(), "mywin","height=350,width=626,status=yes,toolbar=no,menubar=no,location=no");
    }
    
    /*
     * Returns true if the user is logged in. False otherwise
     * We assume the user is logged in of the facebook session
     * has a value.
     */
    this.isLoggedIn = function()
    {
        if (this.session == undefined || this.session == "")
            return false;
            
        return true;
    }

    /*
     * Returns a session key by making an api call to facebook using
     * the code entered by the user as an argument. We perform basic
     * validation before making the auth.getSession call.
     * The newly created session is stored in the gadget's settings.
     * The json response also includes the user's id and a new secret
     * key. Both secret key and uid are stored in the gadget's settings.
     */
    this.createSession = function(token)
    {
        Util.debug("createSession: token(as entered by user)=" + token);
        
        // token is required
        if (token == undefined || token == "")
            Util.error("createSession", Util.getMessage("CodeRequiredError"));

        // token length must be 6
        if (token.length != GadgetSetting.AuthenticationTokenLength)
            Util.error("createSession", Util.getMessage("InvalidCodeError"));
        
        // token must be alphanumeric
        if (Util.isAlphanumeric(token) == false)
            Util.error("createSession", Util.getMessage("InvalidCodeError"));

        this.token = token;
        Util.setSetting("token", this.token);
        Util.debug("createToken: token="+this.token);
        
        // We make the api call to facebook to get the session and uid and new secret
        var response = self.apiCall("facebook.auth.getSession").responseContent;
        
        // We retrieve and store the session
        this.session = response.session_key;
        
        if (this.session == "")
            Util.error("createSession", "response did not contain session key response="+response);
        
        Util.setSetting("session", this.session);
        Util.debug("createSession: session="+this.session);
        
        // We retrieve and store the uid
        this.uid = response.uid;
        Util.setSetting("uid", this.uid);
        Util.debug("createSession: uid="+this.uid);
        
        // We retrieve and store the new secret key
        this.secret = response.secret;
        Util.setSetting("secret", this.secret);
        Util.debug("createSession: New secret="+this.secret);

        return this.session;
    }
    
    /*
     * Clears the state of the gadget. if callBack is provided
     * it will be called when the state is cleared.
     */
    this.clearState = function(callBack)
    {
        Util.removeSetting("session");
        this.session = "";
        
        Util.removeSetting("token");
        this.token = "";
        
        Util.removeSetting("secret");
        this.secret = GadgetSetting.Secret;
        
        Util.removeSetting("uid");
        this.uid = "";
        
        Util.removeSetting("photoUploadAlbumLink");
        this.photoUploadAlbumLink = "";
    
        Util.removeSetting("photoUploadAlbumName");
        this.photoUploadAlbumName = "";
        
        Util.debug("clearState: State cleared.");
        
        if (callBack != undefined && callBack != "")
            eval(callBack);
    }
    
    /*
     * Restores the gadget state from the settings.
     * An error is thrown if throwException is set to true, and no session
     * is defined.
     */
    this.restoreState = function(throwException)
    {
        if (this.session != undefined && this.session != "")
            return this;
            
        this.token = Util.getSetting("token");
        this.session = Util.getSetting("session");
        this.uid = Util.getSetting("uid");
        tmpSecret = Util.getSetting("secret");
        
        if (tmpSecret != "")
            this.secret = tmpSecret;
            
        this.photoUploadAlbumLink = Util.getSetting("photoUploadAlbumLink");
        this.photoUploadAlbumName = Util.getSetting("photoUploadAlbumName");
          
        if (throwException != undefined && throwException == true && (this.session == undefined || this.session == ""))
            Util.error("restoreState", "A session is required for this method.");
            
        return this;
    }
    
    /*
     * The GadgetSetting class defines several FQL queries for retrieving photos.
     * Each query is is identified by a unique name. This function finds the FQL
     * query by name and exectutes the query.
     * This function returns an array of photo objects matching the FQL query.
     * An empty array is returned if no photos match the FQL.
     * The gadget limits the size of photos to GadgetSetting.MaximumPhotoCount in case
     * the resulf of the FQL exceeds GadgetSetting.MaximumPhotoCount. The performance of
     * this call depends on how many photos are returned by facebook.
     * This function requires a session.
     */
    this.getPhotosByName = function(name)
    { 
        if (name == undefined || name == "")
            Util.error("getPhotosByName", "a name is required for this function");
        
        Util.debug("getPhotosByName: name="+name);
        
        this.restoreState(true);
        
        // We retrieve the FQL matching the name provided
        // As an argument
        var fql = self.getFql(name);

        // We exit here if no FQL is found
        if (fql == "")
            Util.error("getPhotosByName", "Unable to find a matching FQL using \""+ name + "\"");
        
        // We make the FQL API call to retrieve the results
        return self.executePhotosFql(fql);
    }

    /*
     * Returns a notification object for the current user.
     * This function uses the notifications.get facebook
     * API call.
     * This function requires a session.
     */
    this.getNotification = function()
    {
        this.restoreState(true);

        return new Notification(self.apiCall("facebook.notifications.get").responseContent);
    }
    
    /*
     * Returns the id of the current facebook user.
     * This function requires a session.
     */
    this.getLoggedInUserId = function() 
    {
        this.restoreState(true);
        
        return self.apiCall("facebook.users.getLoggedInUser").responseContent;
    }
    
    /*
     * Uploads a photo based on the info in the PhotoUploadItem instance.
     * This function also updates the responseContent, errorCode, errorMessage,
     * uploaded, and processed properties based on the result of the photo upload
     * API Call.
     * This function requires a session.
     */
    this.uploadPhotoItem = function(file)
	{   
        // We make sure we have a session
	    this.restoreState(true);
	    
	    // File is required for this operation
	    if (file == undefined)
	        Util.error("uploadPhotoItem", "file must be defined for this call");
        
        var response = null;
        
        // We exit here if the file should not be uploaded
        // due to a local validation error
		if (file.toBeUploaded == false)
			return;

		try
		{
		    // We upload the the photo as a binary file using photos.upload
			response = self.uploadBinaryFile(file);

			// We set responseContent, errorCode, and errorMessage properties
			// of file
			file.responseContent = response.responseContent;			
			file.errorCode = response.getErrorCode();
			file.errorMessage = response.getErrorMessage();
				
			Util.debug("uploadPhotoItem: ErrorCode="+file.errorCode+" ErrorMessage="+file.errorMessage);

            // We mark whether the file was uploaded or not
            // based on the error code and message in the response
			if (file.errorCode=="" && file.errorMessage == "")
				file.uploaded = true;
			else
				file.uploaded = false;
				
		    // get album's link and name
		    if (this.photoUploadAlbumLink == "")		        
                self.getAlbumLink(response.responseContent.pid); 
		}
		catch (ex)
		{
		    // We record any exception in the file object
		    // itself. All exceptions are handled.
		    
		    Util.logErrorMessage("uploadPhotoItem", "HANDLED: "+ex);
		    
            if (response == undefined || response == "")
		    {
			    file.responseContent = "";
			    file.errorCode = -1;
			    file.errorMessage = ex;
			    file.uploaded = false;
		    }
			else
			{
			    // We append our own message if facebook returns
			    // an empty response.
                if (response.length == 0)
                {
                    file.responseContent = "";
                    file.errorCode = "99";
                    file.errorMessage = Util.getMessage("NoResponseRecievedDuringPhotoUploadError");
                    file.uploaded = false;
                }
			    else
			    {
				    file.responseContent = response.responseContent;
				    file.errorCode = response.getErrorCode();
                    file.errorMessage = response.getErrorMessage();

				    if (file.errorCode == "" && file.errorMessage == "")
					    file.uploaded = true;
				    else
					    file.uploaded = false;
			    }
			}
		}
	}

    /*
     * Returns the facebook URL of the page used to get the token code.
     */
	self.getAuthenticationTokenUrl = function()
    {
        return GadgetSetting.AuthenticationTokenUrl + "?v="+GadgetSetting.ApiVersion + "&api_key="+GadgetSetting.ApiKey;
    }
    
    /*
     * Creates an MD5 signature using the provided argument.
     * Before generating the key, this function sorts the parameters
     * alphabetically as required by the facebook API.
     */
    self.createSignature = function(parameters)
    {   
        parameters.sort();
        
        var request = "";
        
        for (var i=0; i<parameters.length; i++)
            request += parameters[i];
        
        request += this.secret;
        
        return Util.getMD5EncodedString(request);
    }
    
    /*
     * Returns the the link and name properties of the album to which photoId belongs.
     * This is done by making a photos.getAlbum API call to Facebook with
     * photoId as an argument. In this case we assume the album must exist
     * since this function is called right after a photo is uploaded using
     * photos.upload. The result is stored in the gadget settings.
     */
	self.getAlbumLink = function(photoId)
    {
        var response = self.apiCall("facebook.photos.getAlbums", new Array("pids="+photoId)).responseContent;
        
        this.photoUploadAlbumLink = response[0].link;
        Util.setSetting("photoUploadAlbumLink", this.photoUploadAlbumLink);
        Util.debug("getAlbumLink: photoUploadAlbumLink="+this.photoUploadAlbumLink);
        
        this.photoUploadAlbumName = response[0].name;
        Util.setSetting("photoUploadAlbumName", this.photoUploadAlbumName);
        Util.debug("getAlbumLink: photoUploadAlbumName="+this.photoUploadAlbumName);
    }
    
    /*
     * Returns the FQL query from GadgetSetting.PhotoOptions by name.
     * Each query has a unique name.
     * The list of queries is defined as JSON array in
     * GadgetSetting.PhotoOptions
     * An error is thrown if the option contains a syntax error.
     * An empty string is returned if no query is found.
     */
    self.getFql = function(name)
    {
        try
        {
            for (var i=0; i<GadgetSetting.PhotoOptions.length; i++)
                if (GadgetSetting.PhotoOptions[i].name == name)
                    return GadgetSetting.PhotoOptions[i].fql;
            
            return "";
        }
        catch(ex)
        {
            Util.error("getFql", "Cannot parse GadgetSetting.PhotoOptions exception="+ex);
        }
    }
    
    /*
     * Executes the FQL statement and returns the result as an array
     * of Photo objects.
     * Before executing the FQL, we look for any keyword and repace it
     * with its corresponding FQL elements. We support 2 types of keywords
     *
     * photos.properties is replaced with the PhotoProperties defined in GadgetSetting
     * user.id is replaces with the id of the current user
     *
     * This function requires a session.
     */
    self.executeFql = function(query)
    {
        this.restoreState(true);
        
        var fql = query.replace(/@photo.properties/g, GadgetSetting.PhotoProperties);
        
        fql = fql.replace(/@user.id/g, this.uid);

        return self.apiCall("facebook.fql.query", new Array("query="+fql));
    }
    
    /*
     * Executes the FQL provided as an argument and returns the result
     * as an array of Photo objects.
     */
    self.executePhotosFql = function(query)
    {
        var result = new Array();
        var photos = self.executeFql(query).responseContent;
         
        if (photos == undefined)
            return result;
            
        for (var i=0; i<photos.length; i++)
        {
            var photo = new Photo(photos[i]);
            result.push(photo);
            
            if (result.length == GadgetSetting.MaximumPhotoCount)
            {
                Util.debug("executePhotosFql truncating list of " + photos.length + " photos to " + GadgetSetting.MaximumPhotoCount);
                break;
            }
        }

        Util.debug("photos.length="+result.length);

        return result;
    }
    
    /*
     * Returns the url for making a specific API call.
     * Before generating the key, this function sorts the parameters
     * alphabetically.
     */
    self.createApiUrl = function(method, additionalParameters)
    {   
        var parameters = new Array("method="+method);
        
        parameters.push("v="+GadgetSetting.ApiVersion);
        parameters.push("api_key=" + GadgetSetting.ApiKey);
        
        if (GadgetSetting.ApiFormat != "")
            parameters.push("format=" + GadgetSetting.ApiFormat);

        if (method != "facebook.auth.getSession")
        {
            parameters.push("session_key="+ this.session);
            parameters.push("call_id=" + Util.getSecondsSince1970());
        }
        else
        {
            if (method == "facebook.auth.getSession")
                parameters.push("auth_token=" + this.token);
        }
            
        if (additionalParameters != undefined)
        {
            for (var i=0; i<additionalParameters.length; i++)
                parameters.push(additionalParameters[i]);
        }
        
        parameters.push("sig="+self.createSignature(parameters));
        
        return GadgetSetting.BaseUrl + "?" + Util.buildQueryString(parameters);
    }
    
    /*
     * Executes the REST API call by posting an HTTP message to facebook's
     * API Server. This function first builds the URL using the provided
     * arguments and then makes an AJAX call. The response is process to check
     * for any error messages return from facebook. This function will throw
     * an error is the AJAX call fails.
     */
    self.apiCall = function(method, parameters) 
    {
        Util.debug("apiCall: method="+method);

        var response = AjaxUtil.doPost(self.createApiUrl(method, parameters), GadgetSetting.ApiFormat);
        
        if (response.hasResponse() == false)
            Util.error("apiCall", "No response received");
            
        return self.processError(response, method, parameters);
    }

    /*
     * This function is called after each API call to check for
     * any error message in the response. We only have special
     * handling for 2 error messages, 102 Session timeout, and 4
     * maximum number of requests reached. For all other errors
     * we just throw an error for the caller to catch.
     */
    self.processError = function(response, method, parameters)
    {
        var errorCode = response.getErrorCode();
        
        // We exit here if response has no error
        if (errorCode == "")
            return response;
            
        var errorMessage = response.getErrorMessage();
        
        Util.log("processError: errorMessage="+errorMessage + " errorCode=" + errorCode);
        
        // Handle Session timeout
        if (errorCode == "102")
        {
            Util.log("Session timed out. Clearing State");
            this.clearState();
            return;
        }
        
        // Handle error 103
        if (errorCode == "103")
        {
            Util.log("processError: cannot make more than one call in the same millisecond. Pausing 10 millseconds and retrying");
            Util.sleep(100);
            return self.apiCall(method, parameters);
        }
        
        // Handle application has reached the maximum number of requests allowed error
        if (errorCode == "4")
        {
            if (this.delay >= GadgetSetting.MillisecondsBeforeRetryLimit)
            {
                this.delay = GadgetSetting.MillisecondsBeforeRetry;
                Util.debug("processError: reached request limit. Throwing exception ");
                Util.error("processError", errorMessage + " (" + errorCode + ")");
            }
            
            Util.log("processError: reached request limit. Pausing " + this.delay + " millseconds and retrying");
            Util.sleep(GadgetSetting.MillisecondsBeforeRetry);
            this.delay = this.delay + 1000;
            return self.apiCall(method, parameters);
        }
        
        // We get here, we need to throw an error
        // since we're not handling the exception
        Util.error("processError", errorMessage + " (" + errorCode + ")");
    }
    
    /*
     * Uploads a file to facebook.
     */
    self.uploadBinaryFile = function(file)
    {
        var response = Util.uploadBinaryFile(self.createApiUrl("facebook.photos.upload"), file.path, file.mimeType);
        
        if (response.length == 0)
            throw Util.getMessage("NoResponseRecievedDuringPhotoUploadError");
			 
        // We convert to JSON since in this specific case it's not done in
        // the VBScript function
        response.responseContent = Util.getJSON(response.responseContent);
			
        var errorCode = response.getErrorCode();
        var errorMessage = response.getErrorMessage();
         
        // We exit here if response has no error
        if (errorCode == "")
            return response;
            
        Util.debug("uploadBinaryFile: errorMessage="+errorMessage + " errorCode=" + errorCode);
        
        // Handle Session timeout
        if (errorCode == "102")
        {
            Util.log("uploadBinaryFile: Session timed out. Clearing State");
            this.clearState();
            return response;
        }
        
        // Handle error 103
        if (errorCode == "103")
        {
            Util.log("uploadBinaryFile: cannot make more than one call in the same millisecond. Pausing 10 millseconds and retrying");
            Util.sleep(100);
            return self.uploadBinaryFile(file);
        }
        
        // Handle application has reached the maximum number of requests allowed error
        if (errorCode == "4")
        {
            if (this.delay >= GadgetSetting.MillisecondsBeforeRetryLimit)
            {
                this.delay = GadgetSetting.MillisecondsBeforeRetry;
                Util.debug("uploadBinaryFile: reached request limit. Throwing exception ");
                Util.error("uploadBinaryFile", errorMessage + " (" + errorCode + ")");
            }
            
            Util.log("uploadBinaryFile: reached request limit. Pausing " + this.delay + " millseconds and retrying");
            Util.sleep(GadgetSetting.MillisecondsBeforeRetry);
            this.delay = this.delay + 1000;
            return self.uploadBinaryFile(file);
        }

        return response;
    }
}
