 /*
 +---------------------------------------------------------------------------+
 | Facebook Microsoft Vista Sidebar Gadget                                   |
 +---------------------------------------------------------------------------+
 | Copyright (c) 2007 Facebook, Inc.                                         |
 | All rights reserved.                                                      |
 |                                                                           |
 | Redistribution and use in source and binary forms, with or without        |
 | modification, are permitted provided that the following conditions        |
 | are met:                                                                  |
 |                                                                           |
 | 1. Redistributions of source code must retain the above copyright         |
 |    notice, this list of conditions and the following disclaimer.          |
 | 2. Redistributions in binary form must reproduce the above copyright      |
 |    notice, this list of conditions and the following disclaimer in the    |
 |    documentation and/or other materials provided with the distribution.   |
 |                                                                           |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR      |
 | IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES |
 | OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.   |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,          |
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  |
 | NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, |
 | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     |
 | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       |
 | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF  |
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.         |
 +---------------------------------------------------------------------------+
 | Developer: Amine Nebri - facebook id anebri@yahoo.com                     |
 +---------------------------------------------------------------------------+
*/

/*
 * PhotoUploadItem class represents all the info related to processing a single
 * image for upload including the path, size, results of the upload, etc.
 * The class also implements validation rules that need to be applied to the file
 * before it is uploaded to facebook.
 */
function PhotoUploadItem(path, name, type, size)
{
	this.path = path; // path of the image to be uploaded.
	this.type = type; // type of the image to be uploaded.
	this.size = size; // Size in bytes of the image to be uploaded.
	this.name = name; // Name of the image to be uploaded.
	this.mimeType = "application/upload"; // Mime type of the image to be uploaded. Initial value is set to application/upload.
	this.processed = false; // Indicates wheather the image was processed or not.
	this.uploaded = false; // Indicates wheather the file was successfully uploaded or not.
	this.errorCode = ""; // error code returned by facebook.
	this.responseContent = ""; // response returned by facebook.
	this.errorMessage = ""; // error message. Could either be a local message or a message returned by facebook.
	this.toBeUploaded; // Indicates wheather the file should be uploaded or not. Files that fail the local validation will not be uploaded.
	this.canceled = false; // Indicated wheather the upload operation was canceled or not.

	/*
	 * Applies validation rules to the item to ensure it won't be
	 * rejected by the photo upload API. Validation rules include
	 * - Image Type (only PNG, GIG, JPEG) are allowed
	 * - File Size (must be <= 4.5 MB)
	 * The files that fail the validation will not be uploaded.
	 */
	this.eval = function()
	{
		this.errorMessage = Util.getMessage("InvalidImageTypeError");
		this.toBeUploaded = false;
		this.processed = false;

		for (var i=0; i<GadgetSetting.ValidUploadableImageTypes.length; i++)
		{
			if (GadgetSetting.ValidUploadableImageTypes[i].type == this.type)
			{
				this.errorMessage = "";
				this.mimeType = GadgetSetting.ValidUploadableImageTypes[i].mimeType;
				break;
			}
	    }
	    
		if (this.errorMessage != "")
		{
			this.processed = true;
			return this;
		}
	    
		this.toBeUploaded = true;
		this.processed = false;
		this.errorMessage = "";
        
		if (this.size > GadgetSetting.MaximumPhotoFileSizeInBytes)
		{
			this.errorMessage = Util.getMessage("ImageTooBigError");
			this.toBeUploaded = false;
			this.processed = true;
			return this;
		}
	    
		return this;
	}
    
	/*
	 * Returns an info string with all the properties of the class instance.
	 */
	this.stringValue = function()
	{
		return "processed="+this.processed+",errorMessage=["+this.errorMessage+"],canceled="+this.canceled+"],uploaded=["+this.uploaded+"],name=["+this.name+"],type=["+this.type+"],mimeType=["+this.mimeType+"],size=["+this.size+"],toBeUploaded=["+this.toBeUploaded+"],path=["+this.path+"],errorCode=["+this.errorCode+"],responseContent=["+this.responseContent+"]";
	}
}

/*
 * PhotoUploadItemSummary class represents the results of the upload process
 * including a collection of PhotoUploadItem objects. PhotoUploadItemSummary
 * is invoked right after the user drops photos inside the gadget main window.
 * We recursively look for all files inside any folder selected by the user.
 * Each file is then validated. Once all files validated, the upload process
 * then starts one file at a time using a timer to allow the user interface to
 * show progress.
 * we use callback functions to inform the user interface when the process starts,
 * when a file is about to be uploaded, and when the process is completed.
 */
PhotoUploadItemSummary = new function()
{
    var self = this; // Used to implement private functions
    this.items = new Array(); // Collection of PhotoUploadItem objects to be uploaded.
    this.timer; // timer used for the upload process.
	this.isProcessingInProgress = false; // Indicates weather an upload is in progress or not.
	this.startCallBack; // call back function that gets called before the files are processed.
	this.progressCallBack; // call back function that gets called  before each file is uploaded/processed.
	this.endCallBack; // Call back function thagt gets called when the last file is uploaded/processed.
	this.canceled = false; // Indicate wheather the upload process was canceled by the user.
	this.summaryMessage = ""; // Shows the overall error message if any.

	/*
	 * This function is called after the user drag and drops files inside the gadget
	 * main window to start the upload process.
	 * we use callback functions to inform the user interface when the process starts,
	 * when a file is about to be uploaded, and when the process is completed.
	 */
	this.startUpload = function(startCallBack, progressCallBack, endCallBack)
	{
		FacebookAPI.restoreState(true);
		
		try
		{
			this.canceled = false;	// reset to default value
			this.items = new Array(); // reset files
		
		    // We exit here if the function is called with
		    // a pending file upload
			if (self.hasUnprocessedItems() == true)
				Util.error("startUpload", "Drag and Drop in progress. Please try again.");
		
		    // We update the call back properties
			this.startCallBack = startCallBack;
			this.progressCallBack = progressCallBack;
			this.endCallBack = endCallBack;
	
	        // We load the items collection from the
	        // drag and drop event
			self.loadFromFileDropEvent();

            // We call the start call back
			eval(this.startCallBack + "()");
			
			// We start the process if there's at least
			// one file that needs to be uploaded
			if (self.hasUnprocessedItems())
				this.onProgress(true);
			else // We call the end call back function
				eval(this.endCallBack + "(this)");
		}
		catch (ex)
		{
			Util.error("startUpload", ex);
		}
	}
    
    /*
     * Returns the number of files maked as uploaded.
     */
    this.itemsUploadedCount = function()
    {      
        var count = 0;
        
        for (var i=0; i<this.length(); i++)
            if (self.itemAt(i).uploaded == true)
                count++;
        
        return count;
    }
    
    /*
     * Returns the length of the items array.
     */
    this.length = function()
    {
        if (this.items == undefined)
            return 0;
        
        return this.items.length;
    }
    
    /*
     * Returns the a collection of errors summarizing the results
     * of the upload process. The errors are grouped by error message
     */
    this.uploadErrors = function()
    {
        var result1 = new Array();
        
        // We build a set of error messages
        for (var i=0; i<this.length(); i++)
        {
            var item = self.itemAt(i);
            
            if (item.errorMessage != "")
                Util.pushIfMissing(result1, item.errorMessage);
        }

        // We pre-populate the error messages
        var result = new Array();

        for (var i=0; i<result1.length; i++)
        {
            var object = new Object();
            
            object.error = result1[i];
            object.files = new Array();
            
            result.push(object);
        }
 
        // We populate the list of files
        for (var i=0; i<this.length(); i++)
        {
            var item = self.itemAt(i);
            
            if (item.errorMessage != "")
            {
                var index = Util.getIndexOf(result1, item.errorMessage);

                result[index].files.push(item);
            }
        }
         
        if (Util.isDebug())
        {
            Util.debug("uploadErrors: count="+result.length);
              
            for (var i=0; i<result.length; i++)
            {
                for (var j=0; j<result[i].files.length; j++)
                    Util.debug("uploadErrors: j="+j+" file="+result[i].files[j].stringValue());
            }
        }
        
        return result;
    }
    
    /*
     * Called to upload a single file to facebook using the photo.upload API Call.
     * if the pending photo upload is full or has too many unapproved photos then
     * we cancel the remaining files.
     */
    this.uploadFile = function(file, index)
    {
        FacebookAPI.uploadPhotoItem(file);

		if (file.errorCode == "321" || file.errorCode == "325")
		{
			this.summaryMessage = file.errorMessage;
			self.cancelItems(index, this.summaryMessage);
			self.printSummary();
			eval(this.endCallBack + "(this)");
		}
		else
			file.processed = true;
	}

    /*
     * This function is called during the photo upload process
     * for each file.
     */
    this.onProgress = function(flag)
	{
	    self.printSummary();
	
		try
		{
			clearTimeout(this.timer);
			
			if (this.isProcessingInProgress)
			{
				Util.debug("onProgress - busy (inflight)");
				this.timer = setTimeout("PhotoUploadItemSummary.onProgress(true, false)", 0);
				return;
			}
	
			this.isProcessingInProgress = true;
            
			if (self.hasUnprocessedItems())
			{
			    var index = self.getIndexOfFirstUnprocessedItem();
			    
			    if (this.canceled)
			    {
			        Util.debug("onProgress: cancelled!");
			        this.summaryMessage = Util.getMessage("UserCancellationError");
                    self.cancelItems(index, this.summaryMessage);
                    self.printSummary();
                    eval(this.endCallBack + "(this)");
			    }
			    else
			    {
			        var file = self.getFirstUnprocessedItem();
    			    
			        if (file == undefined)
			            Util.error("onProgress", "No Unprocessed file found");
			            
                    Util.debug("onProgress: processing file = " + file.stringValue());
    	
                    if (flag)
                    {
                        eval(this.progressCallBack + "(self.processedItemsCount(), this.length(), file)");
                        this.timer = setTimeout("PhotoUploadItemSummary.onProgress(false)", Math.max(file.size/900, 1000));
                    }
                    else
                    {
                        PhotoUploadItemSummary.uploadFile(file, index);
                        this.timer = setTimeout("PhotoUploadItemSummary.onProgress(true)", 0);
					}
                }
			}
			else
			{
                Util.debug("onProgress: processing done!");
				self.printSummary();
		    	eval(this.endCallBack + "(this)");
			}
		}
		catch (ex)
		{
		    Util.error("onProgress", ex);
		}
		
		this.isProcessingInProgress = false;
	}
	
    /*
     * Returns the item at the specified index.
     */
    self.itemAt = function(index)
    {
        return this.items[index];
    }
    
	/*
	 * Adds a new item to the items collection. item is also
	 * validated right after it is added.
	 */
	self.addItem = function(item)
	{
		this.items.push(item);
		item.eval();
	}
	
    /*
     * Populates the items collection using from the drag and drop event.
     * We will recursively search for files if the selection includes folders.
     * We also check to make sure the number of files does not exceed the maximum
     * number of files allowed as defined in GadgetSetting.MaximumPhotoUploadCount and
     * it does we only upload up to MaximumPhotoUploadCount and cancel the rest.
     */
    self.loadFromFileDropEvent = function()
    {
        var i=0;
        
        this.items = new Array();
        
        // We load the files into the items collection
        while (System.Shell.itemFromFileDrop(event.dataTransfer, i))	        
            self.processItem(System.Shell.itemFromFileDrop(event.dataTransfer, i++));
        
        // We make sure the user does not attempt to upload
        // more file than allowed.
        if (self.toBeUploadedCount() > GadgetSetting.MaximumPhotoUploadCount)
        {
            this.summaryMessage = Util.getMessage("FileUploadLimitError");
            self.cancelItems(0, this.summaryMessage);
        }

        self.print();
    }
    
    /*
     * Called from the loadFromFileDropEvent function to populate
     * the collection of files to be uploaded. This function has the
     * recursion code to look for files inside each folder.
     */
    self.processItem = function(item)
	{
	    Util.debug("processItem: path="+item.path);
	    
	    // If it's a link we find the referenced file.
	    if (item.isLink)
	    {        
	        if (item.link.length > 0)
                return self.processItem(System.Shell.itemFromPath(item.link));

            return;
	    }
	    
	    // If it's a folder we recursively find all the files
		if (item.isFolder)
		{
			var folder = item.SHFolder;
			
            for (var i=0; i<folder.Items.count; i++)
			    self.processItem(folder.Items.item(i));
		}
		else
            self.addItem(new PhotoUploadItem(item.path, item.name, item.type, item.size));
	}
    
    /*
     * Iterates through the items collection and marks all
     * files starting at index startIndex as canceled using
     * the provided error message.
     */
    self.cancelItems = function(startIndex, errorMessage)
    {
        for (var i=startIndex; i<this.length(); i++)
        {
            var item = self.itemAt(i);
            
            item.processed = true;
            item.toBeUploaded = false;
            item.errorMessage = errorMessage;
            item.canceled = true;
        }
    }
	
	/*
	 * Prints a debug summary string with all the properties of the
	 * class instance excluding the items to be uploaded.
	 */
    self.printSummary = function()
    {
        Util.debug("fileCount="+this.length()+" processed="+self.processedItemsCount()+" uploaded="+this.itemsUploadedCount()+ " toBeUploaded="+self.toBeUploadedCount());
    }
    
	/*
	 * Prints a debug string with all the properties of the class instance including all the files.
	 */
    self.print = function(title)
    { 
        var result = "";
        
        if (title != undefined)
            Util.debug(title);
        
        self.printSummary();
        
        for (var i=0; i<this.length(); i++)
            Util.debug((i+1) + "/" + this.length() + " "+ self.itemAt(i).stringValue());
    }
    
    /*
     * Returns the first unprocessed item.
     */
    self.getFirstUnprocessedItem = function()
    {
        for (var i=0; i<this.length(); i++)
            if (self.itemAt(i).processed == false)
                return self.itemAt(i);
        
        return null;
    }
    
    /*
     * Returns true if there's at least one file marked
     * unprocessed. false otherwise.
     */
    self.hasUnprocessedItems = function()
    {
        if (self.processedItemsCount() < this.length())
            return true;
        
        return false;
    }
    
    /*
     * Returns the index of the first unprocessed item.
     */
    self.getIndexOfFirstUnprocessedItem = function()
    {
        for (var i=0; i<this.length(); i++)
            if (self.itemAt(i).processed == false)
                return i;
        
        return -1;
    }
    
    /*
     * Returns the number of files marked as processed.
     */
    self.processedItemsCount = function()
    {
        var count = 0;
        
        for (var i=0; i<this.length(); i++)
            if (self.itemAt(i).processed == true)
                count++;
        
        return count;
    }
    
    /*
     * Returns the number of files marked to be uploaded.
     */
    self.toBeUploadedCount = function()
    {
        var count = 0;
        
        for (var i=0; i<this.length(); i++)      
            if (self.itemAt(i).toBeUploaded == true)
                count++;
        
        return count;
    }
}