 /*
 +---------------------------------------------------------------------------+
 | Facebook Microsoft Vista Sidebar Gadget                                   |
 +---------------------------------------------------------------------------+
 | Copyright (c) 2007 Facebook, Inc.                                         |
 | All rights reserved.                                                      |
 |                                                                           |
 | Redistribution and use in source and binary forms, with or without        |
 | modification, are permitted provided that the following conditions        |
 | are met:                                                                  |
 |                                                                           |
 | 1. Redistributions of source code must retain the above copyright         |
 |    notice, this list of conditions and the following disclaimer.          |
 | 2. Redistributions in binary form must reproduce the above copyright      |
 |    notice, this list of conditions and the following disclaimer in the    |
 |    documentation and/or other materials provided with the distribution.   |
 |                                                                           |
 | THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR      |
 | IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES |
 | OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.   |
 | IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,          |
 | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  |
 | NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, |
 | DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     |
 | THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       |
 | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF  |
 | THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.         |
 +---------------------------------------------------------------------------+
 | Developer: Daniel Ruspini - facebook id dan@ruspini.com                   |
 +---------------------------------------------------------------------------+
*/

/*
 * SlideShow is the function literal controlling facebook slideshow UI.
 */
SlideShow = new function() {
	this.photosArray = new Array();				// image urls
	this.photosAPIMapping = new Object;			// image store
	this.imageStoreType;						// current selected image store	(from settings)
	this.delay;									// time in between transitions from slide to slide (from settings)
	this.transition;							// transition type (from settings)
	this.stopped = false;						// slideshow is stopped
	this.viewingImgNode = "";					// currently viewed image node (either 'img1' or 'img2')
	this.imgNodeSuffix = 1;						// suffix of currently viewed image node (ie. 1 or 2)
	this.timer;									// timer used for delay
	this.urlsIdx = 0; 							// where in the images we are
	this.doShuffle = true;						// do we randomly sort the photosArray?
	this.isCropped;								// do we crop the images (from settings)
	this.maxDimensions = { width:0, height:0 };	// maximum dimensions of slideshow viewport - set when dock and undock events occur
	this.slideshowCaptionNode;					// node for slideshow caption
	this.noPhotosDelay = 600000;					// 10 minutes delay - time in between checking Facebook for images when there are no photos to display.
	
	/*
	 *  Sets variables before the SlideShow ever starts, including settings and references to nodes.
	 */
	this.initialize = function(settings) {
		this.delay = settings.slideshowDelay;
		this.imageStoreType = settings.slideshowImageStore;
		this.transition = settings.slideshowTransition;
		
		this.photosAPIMapping.typeMe = "PhotosOfMe";
		this.photosAPIMapping.typeFriendsAlbums = "PhotosFromMyFriendsAlbums";
		this.photosAPIMapping.typeMyAlbums = "PhotosFromMyAlbums";
		this.photosAPIMapping.typeMeAndFriendsAlbums = "PhotosFromMeAndMyFriendsAlbums";
		
		this.viewingImgNode = document.getElementById("img1");
		this.slideshowCaptionNode = document.getElementById("slideshow-caption");
	}
	
	/*
	 *  Retrieves image URLs From Facebook API.
	 *  If there are no images (for a particular image store), display the "No photos" pane,
	 *  otherwise start SlideShow.
	 *
	 *  Like all API calls we check if there is an internet connection, 
	 *  otherwise go into 'No Internet' mode which is handled inside the Manager.
	 */
	this.loadPhotos = function() {
		if (Manager.hasInternetConnection()) {
			this.photosArray = FacebookAPI.getPhotosByName(this.photosAPIMapping[this.imageStoreType]);

			if (this.photosArray.length == 0) {
				this.showNoPhotosPane();
				setTimeout("SlideShow.loadPhotos()", this.noPhotosDelay);
			} else {
				if (this.doShuffle && this.photosArray.length > 1) {
					this.photosArray.sort(randomSort);
				}
				this.urlsIdx = 0;

				this.hideNoPhotosPane();
				this.start();
			}
		} else {
        	Manager.startPeriodicalConnectionCheck();
        }
	}
	
	/*
	 *  Stops the SlideShow timer, effectively halting the transitions and subsequent API calls.
	 *  Also set the control bar to show that it is in the Paused state.
	 */
	this.stop = function() {
		clearTimeout(this.timer);
		this.stopped = true;
		$('#slideshow-nav').addClass('paused')
		document.getElementById("ss-playPause").title = "Play";
	}
	
	/*
	 *  Initially starts the SlideShow, changing the "stopped" property and running play()
	 *  which renews transitions and subsequent API calls.  
	 *  Also set the control bar to show that it's in a Play state.
	 */
	this.start = function() {
		this.stopped = false;
		$('#slideshow-nav').removeClass('paused')
		document.getElementById("ss-playPause").title = "Pause";
		this.play();
	}
	
	/*
	 *  Readies the next viewable image node by settings its source to the URL to be shown.
	 *  The consequence of calling this method is that the onLoad event will be called which
	 *  in turn will call play() again (after the delay).  
	 *  The name of this function is play() because it effectively pushes the "play" button
	 *  on the SlideShow.
	 *  We also want to check photo length here, because of the edge case that someone is 
	 *  viewing no photos, does an upload and then continues their slideshow.  Technically
	 *  going back to your slideshow does not go through the loadPhotos() function, as it shouldn't,
	 *  so we need to check here in case that has happened.
	 */
	this.play = function() {
		clearTimeout(this.timer);
		if (this.photosArray.length == 0) {
			this.loadPhotos();
		}
		var newImgNode = this.nextImgNode();
		newImgNode.src = this.photosArray[this.urlsIdx].bigSource;
	}
	
	/*
	 *  Event called when image node has fully loaded the URL.
	 *  This method runs everything that happens in between transitions**:
	 *  (** everything between apply() and play() happens during the specified transition)
	 *      - Sets the next upcoming caption
	 *      - Sends the next image to a utility function that scales and centers the image
	 *      - Then displays this image node, while hiding the previously visible image node.
	 *
	 *  After the transition, if the SlideShow is not stopped, we either use the timer 
	 *  to run another play(), or if we're out of images we call loadPhotos() to go back
	 *  to the Facebook API to retrieve a fresh set of images.
	 */
	this.onLoadImage = function(imgNode) {
		document.getElementById("imgWrapper").filters(this.transition).apply();

		// ready caption
		this.slideshowCaptionNode.innerHTML = this.photosArray[this.urlsIdx].caption;
		if (this.photosArray[this.urlsIdx].caption == "") {
			this.slideshowCaptionNode.style.display = "none";
		} else {
			this.slideshowCaptionNode.style.display = "";
		}
		
		this.viewingImgNode.style.display = "none";
		Util.setStylesBasedOnImageDimensions(imgNode, this.maxDimensions, this.isCropped);
		this.viewingImgNode = imgNode;
		this.viewingImgNode.style.display = "";
		
		document.getElementById("imgWrapper").filters(this.transition).play();
		
		if (!this.stopped) {
			if (this.urlsIdx+1 < this.photosArray.length) {	  // if there are more photos in this set, call again
				this.timer = setTimeout("SlideShow.advanceImageIndex();SlideShow.play()", this.delay);
			} else {
				// Use a timer to call loadPhotos as well, otherwise if a user has only 1 photo it will
				// call loadPhotos() too many times (too quickly).
				setTimeout("SlideShow.loadPhotos()", this.delay);
			}
		}
	}
	
	/*
	 *  Advances the index that keeps track of where we are in the photo array.
	 *  If we've already reached the end of the array, then set it to zero.
	 */
	this.advanceImageIndex = function() {
		this.urlsIdx++;
		if (this.urlsIdx > this.photosArray.length-1) {
			this.urlsIdx = 0;
		}
	}
	
	/*
	 *  Decrease the index that keeps track of where we are in the photo array.
	 *  If we're already at the beginning of the array, go to end of array.
	 */
	this.decreaseImageIndex = function() {
		this.urlsIdx--;
		if (this.urlsIdx < 0) {
			this.urlsIdx = this.photosArray.length-1;
		}
	}
	
	/*
	 *  Event called when user clicks "Next" button on SlideShow control bar.
	 *  Advance the image index and re-call play function.
	 */
	this.onClickNext = function() {
		this.advanceImageIndex();
		this.play();
	}
	
		
	/*
	 *  Event called when user clicks "Previous" button on SlideShow control bar.
	 *  Decrease the image index and re-call play function.
	 */
	this.onClickPrevious = function() {
		this.decreaseImageIndex();
		this.play();
	}
	
	/*
	 *  Event called when user clicks "Pause/Play" button on SlideShow control bar.
	 *  Starts or stops the SlideShow app and controls the UI display.
	 */
	this.onClickTogglePaused = function() {
		if (this.stopped) {
			this.start();
		} else {
			this.stop();
		}
	}
	
	/*
	 *  Event called when user double clicks an image, 
	 *  opening the URL (on Facebook) in a new window.
	 */
	this.openInWindow = function() {
		if (this.photosArray.length > 0) {
			window.open(this.photosArray[this.urlsIdx].link);
		} else {
			event.returnValue = false;
		}
	}
	
	/*
	 *  Event called when user clicks the 'Share' button.
	 *  Opens a new window to Facebook share page.
	 */
	this.share = function() {
	 	window.open("http://www.facebook.com/sharer.php?u=" + encodeURIComponent(this.photosArray[this.urlsIdx].link) + "&t=" + encodeURIComponent(this.photosArray[this.urlsIdx].caption), "Facebook", "toolbar=0,status=0,height=436,width=646,scrollbars=yes,resizable=yes");
	}
	
	/*
	 *  Returns the image node to be used next (ie. the currently non-viewable image node)
	 */
	this.nextImgNode = function() {
		this.imgNodeSuffix = this.imgNodeSuffix == 1 ? 2 : 1;
		return document.getElementById("img" + this.imgNodeSuffix);
	}
	
	/*
	 *  Fades out the image caption
	 */
	this.hideCaption = function() {
		$("#slideshow-caption").fadeTo("normal", 0);
	}
	
	/*
	 *  Fades in the image caption
	 */
	this.showCaption = function() {
		if(this.photosArray[this.urlsIdx].caption != "") {
			$("#slideshow-caption").fadeTo("normal", 0.75);
		}
	}
	
	/*
	 *  Fades out the SlideShow control bar
	 */
	this.hideControls = function() {
		$("#slideshow-controls").fadeTo("fast", 0);
		$("#slideshow-controls-bkgnd").fadeTo("normal", 0);
	}
	
	/*
	 *  Fades in the SlideShow control bar
	 */
	this.showControls = function() {
		$("#slideshow-controls").fadeTo("normal", 1);
		$("#slideshow-controls-bkgnd").fadeTo("fast", 0.75);
	}
	
	/*
	 *  Event called when user mouses over image, showing controls and caption
	 */
	this.onMouseOver = function() {
		if(event.fromElement) return;
		this.showControls();
		this.showCaption();
	}
	
	/*
	 *  Event called when user mouses out of image, hiding controls and caption
	 */
	this.onMouseOut = function() {
		if(event.toElement) return;
		this.hideControls();
		this.hideCaption();
	}
	
	/*
	 *  Event called when user docks the gadget.
	 *  Change maxDimensions; Resize the current viewable image; Hide the favicon (which doesn't fit in the docked state)
	 */
	this.dock = function() {
		this.maxDimensions.width = Manager.dockedDimensions.width - 16;
		this.maxDimensions.height = Manager.dockedDimensions.height - 16;
		this.resizeCurrentImage();
		document.getElementById("ss-favicon").style.display = "none";
	}
	
	/*
	 *  Event called when user undocks the gadget.
	 *  Change maxDimension; Resize the current viewable image; Show the favicon.
	 */
	this.undock = function() {
		this.maxDimensions.width = Manager.undockedDimensions.width - 16;
		this.maxDimensions.height = Manager.undockedDimensions.height - 16;
		this.resizeCurrentImage();
		document.getElementById("ss-favicon").style.display = "";
	}
	
	/*
	 *  Sends the currently viewed image node into the utility function to scale and center the image.
	 *  Called when the user docks and undocks the gadget.
	 */
	this.resizeCurrentImage = function() {
		if (this.photosArray.length > 0) {
			Util.setStylesBasedOnImageDimensions(this.viewingImgNode, this.maxDimensions, this.isCropped);
		}
	}
	
	/*
	 *  Displays the SlideShow container node
	 */
	this.show = function() {
		document.getElementById("slideshow").style.display = "";
	}
	
	/*
	 *  Hides the SlideShow container node
	 */
	this.hide = function() {
		document.getElementById("slideshow").style.display = "none";
	}
	
	/*
	 *  Displays the "You have no photos" container node.
	 *  Hide the two image panes so that when the SlideShow starts again
	 *  we don't see a brief flash of the previous image.
	 */
	this.showNoPhotosPane = function() {
		document.getElementById("no-photos").style.display = "";
		document.getElementById("img1").style.display = "none";
		document.getElementById("img2").style.display = "none";
	}
	
	/*
	 *  Hides the "You have no photos" container node
	 */
	this.hideNoPhotosPane = function() {
		document.getElementById("no-photos").style.display = "none";
	}
	
}

function randomSort() {
    return Math.random() - 0.5;
}